/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * This file is a part of the libzmf project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include <libzmf/libzmf.h>
#include "ZMF4Header.h"
#include "ZMF4Parser.h"
#include "libzmf_utils.h"

namespace libzmf
{

namespace
{

RVNGInputStreamPtr getContentStream(librevenge::RVNGInputStream *input)
{
  if (!input)
    return RVNGInputStreamPtr();

  RVNGInputStreamPtr input_(input, ZMFDummyDeleter());

  if (input_->isStructured())
  {
    if (input_->existsSubStream("content.zmf"))
      input_.reset(input_->getSubStreamByName("content.zmf"));
    else
      return RVNGInputStreamPtr();
  }

  return input_;
}

}

bool ZMFDocument::isSupported(librevenge::RVNGInputStream *const input, Type *const type, Kind *const kind) try
{
  RVNGInputStreamPtr input_ = getContentStream(input);
  if (!input_)
    return false;

  ZMF4Header header;
  if (header.load(input_) && header.isSupported())
  {
    if (type)
      *type = TYPE_DRAW;
    if (kind)
      *kind = KIND_DRAW;
    return true;
  }

  return false;
}
catch (...)
{
  return false;
}

bool ZMFDocument::parse(librevenge::RVNGInputStream *input, librevenge::RVNGDrawingInterface *painter) try
{
  RVNGInputStreamPtr input_ = getContentStream(input);
  if (!input_)
    return false;

  ZMFCollector collector(painter);

  ZMF4Parser parser(input_, &collector);

  return parser.parse();
}
catch (...)
{
  return false;
}

}

/* vim:set shiftwidth=2 softtabstop=2 expandtab: */
